; ##############################################################################
; ##############################################################################
; ##############################################################################
USART_INITIALISIEREN:
	ldi temp,TAKT/(BAUD*16)-1  		; Baudrate 								
	out UBRRL,temp
   	ldi temp, (1<<URSEL)|(3<<UCSZ0)	; Frame-Format: 8 Bit					
    out UCSRC, temp
    sbi UCSRB, RXCIE                ; Interrupt bei Empfang	aktivieren		
    sbi UCSRB, RXEN 				; RX aktivieren (Empfang)      			
	sbi UCSRB, TXEN  				; TX aktivieren                 		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 	INPUT: temp1														
;																		
;	rcall SENDEN	; INP: TEMP1										
;																		
USART_SENDEN_TEMP1:
	WDR 						; Watch-Dog-RESET					
	sbis UCSRA,UDRE  		   	; Warten bis UDR frei             	
	rjmp USART_SENDEN_TEMP1		; Sprung							
	out UDR, temp1				; senden							
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Interruptroutine: wird ausgefhrt sobald ein Byte ber das UART empfangen wurde
; Alle USART Bytes im Hilfspuffer sammeln und anschlieend komplett in den		
; USART-Hauptpuffer schieben													
; Hier kann es zum starken Jittern des Ausgangssignals kommen					
USART_RXD:
	out TCNT0,TIMER0WERT; berwachung / TIMER0 Startwert		
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push ZL
	push ZH
; ------------------------------------------------------------------------------
; BYTE 0 (Vorhut)																
USART_RXD_0:
	; Hilfszeiger laden	
	LDS temp,(adr_USART_HILFS_ZEIGER)
	cpi temp,0
	brne USART_RXD_1
	; VORHUT	
	in temp, UDR				; BYTE einlesen	
	cpi temp,'*'
	breq USART_RXD_0_OK
USART_RXD_0_ERROR:
	rjmp USART_RXD_NO_ZEIGER	; ERROR		
USART_RXD_0_OK:
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 1 																		
USART_RXD_1:
	cpi temp,1
	brne USART_RXD_2
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+0),temp		; im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 2 																		
USART_RXD_2:
	cpi temp,2
	brne USART_RXD_3
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+1),temp		; AH im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 3
USART_RXD_3:
	cpi temp,3
	brne USART_RXD_4
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+2),temp		; AL im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 4
USART_RXD_4:
	cpi temp,4
	brne USART_RXD_5
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+3),temp		; BH im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 5
USART_RXD_5:
	cpi temp,5
	brne USART_RXD_6
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+4),temp		; BL im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 6
USART_RXD_6:
	cpi temp,6
	brne USART_RXD_7
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+5),temp		; im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 7
USART_RXD_7:
	cpi temp,7
	brne USART_RXD_8
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+6),temp		; im SRAM ablegen
	rjmp USART_RXD_FERTIG
; ------------------------------------------------------------------------------
; BYTE 8 																		
USART_RXD_8:
	in temp, UDR		; BYTE einlesen	
	STS(adr_USART_HILFSBUFFER8+7),temp		; im SRAM ablegen		
; ------------------------------------------------------------------------------
PUFFER_CLEAR:
	LDS temp,(adr_USART_HILFSBUFFER8+0)	; AH laden		
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+1)	; AL laden		
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+2)	
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+3)	
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+4)	
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+5)	
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+6)	
	tst temp
	brne PUFFER_NO_CLEAR
	LDS temp,(adr_USART_HILFSBUFFER8+7)	
	tst temp
	brne PUFFER_NO_CLEAR
	; Z-initialisieren				
	ldi ZL, LOW (adr_USART_BUFFER256)
	ldi ZH, HIGH(adr_USART_BUFFER256)
	; SCHLEIFE
	clr temp
PUFFER_CLEAR_S:
	out TCNT0,TIMER0WERT; berwachung / TIMER0 Startwert		
	ST Z+,NULL
	dec temp
	brne PUFFER_CLEAR_S
	; ZEIGER auf NULL setzen		
	STS(adr_USART_HILFS_ZEIGER),NULL
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR		; GELB an				
	rjmp USART_RXD_NO_ZEIGER
PUFFER_NO_CLEAR:
; ------------------------------------------------------------------------------
; Die Daten aus dem Hilfspuffer in den Hauptpuffer bertragen					
	; Z-initialisieren				
	ldi ZL, LOW (adr_USART_BUFFER256)
	ldi ZH, HIGH(adr_USART_BUFFER256)
	LDS temp,(adr_USART_ZEIGER_WRITE)		; Zeiger
	add ZL,temp
	adc ZH,NULL
	subi temp,-8
	STS(adr_USART_ZEIGER_WRITE),temp		; Zeiger
; ------------------------------------------------------------------------------
	LDS temp,(adr_USART_HILFSBUFFER8+0)	; AH laden		
	ST Z+,temp							; AL speichern	
	LDS temp,(adr_USART_HILFSBUFFER8+1)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+2)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+3)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+4)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+5)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+6)
	ST Z+,temp
	LDS temp,(adr_USART_HILFSBUFFER8+7)
	ST Z+,temp
	; ZEIGER auf NULL setzen		
	STS(adr_USART_HILFS_ZEIGER),NULL
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR		; GELB an				
	rjmp USART_RXD_NO_ZEIGER
; ------------------------------------------------------------------------------
USART_RXD_FERTIG:
	LDS temp,(adr_USART_HILFS_ZEIGER)		; Zeiger+1		
	inc temp
	STS(adr_USART_HILFS_ZEIGER),temp
; ------------------------------------------------------------------------------
USART_RXD_NO_ZEIGER:
	pop ZH
	pop ZL
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 		
reti              
; ##############################################################################
; ##############################################################################
; ##############################################################################

